package org.greenstone.gsdl3.gs3build.doctypes;

import java.util.List;
import java.util.ArrayList;

import java.io.PrintWriter;
import java.io.FileWriter;
import java.io.File;
import java.io.IOException;

import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;

public class DocumentList
{
	DocumentInterface []       list;
	int                        size;
	int                        used;
	DocumentIDFactoryInterface idFactory;
	
	public DocumentList(DocumentIDFactoryInterface idFactory)
	{	this.idFactory = idFactory;
		this.list = new DocumentInterface[10];
		this.used = 0;
		this.size = 10;
	}

	public void addDocument(DocumentInterface document)
	{	if (this.used == this.size) {
			this.ensureSize(this.size * 2);
		}
		this.list[this.used] = document;
		DocumentID id = this.idFactory.getNewDocumentID(document);
		document.setID(id);
		this.used ++;
	}

	public DocumentInterface getDocument(int index)
	{	if (index < 0 || index >= this.used)
		{	return null;
		}
		return this.list[index];
	}

	public DocumentID getDocumentID(int index)
	{	if (index < 0 || index >= this.used)
		{	return null;
		}
		return this.list[index].getID();
	}

	public void ensureSize(int size)
	{	DocumentInterface [] newList = new DocumentInterface[size];
		System.arraycopy(this.list, 0, newList, 0, this.size);
		this.list = newList;
		this.size = size;
	}

	public void writeDocuments(File directory)
	{	for (int i = 0; i < this.used; i ++)
		{	try
			{
				File localFile = new File(directory, "Doc"+Integer.toString(i)+".xml");
				FileWriter fileWriter = new FileWriter(localFile);
				PrintWriter writer = new PrintWriter(fileWriter);
				this.list[i].getMETSWriter().writeDocument(this.list[i], writer);
				writer.close();
				fileWriter.close();
			}
			catch (IOException io)
			{	
			}
		}
	}

	public void writeSQLDocuments(GS3SQLConnection connection)
	{	for (int i = 0; i < this.used; i ++)
		{   this.list[i].getSQLWriter().writeDocument(this.list[i], connection);
		}
	}

	public int size()
	{	return this.used;
	}
}
