package org.greenstone.gsdl3.gs3build.metadata;

import java.io.PrintWriter;

import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Iterator;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.w3c.dom.Text;

import org.xml.sax.SAXException;
import org.xml.sax.SAXParseException;


import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;

import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;

public class METSDescriptiveSet
{ Map children;

  public METSDescriptiveSet()
  { this.children = new HashMap();

    METSDescriptive descriptive = new METSDescriptive("DM1", "default");
    this.addDescriptive(descriptive);
  }

  public void addDescriptive(METSDescriptive metadata)
  { this.children.put(metadata.getName(), metadata);
  }

  public METSDescriptive getDescriptive(String name)
  { return (METSDescriptive) this.children.get(name);
  }

  public void addNamespace(String descriptiveName, METSNamespace namespace)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);
    if (descriptive != null)
    { descriptive.addNamespace(namespace);
    }
  }

  public METSNamespace getNamespace(String descriptiveName, String namespace)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);
    if (descriptive != null)
    { return descriptive.getNamespace(namespace);
    }
    return null;
  }

  public METSNamespace getOpenNamespace(String descriptiveName, String namespace)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);

    if (descriptive != null)
    { return descriptive.getOpenNamespace(namespace);
    }
    return  null;
  }

  public void addMetadata(String descriptiveName, String namespace, String label, String value)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);

    if (descriptive != null)
    { descriptive.addMetadata(namespace, label, value);
    }
  }

  public void setMetadata(String descriptiveName, String namespace, String label, String value)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);

    if (descriptive != null)
    { descriptive.setMetadata(namespace, label, value);
    }
  }

  public List getMetadata(String descriptiveName, String namespace, String label)
  { METSDescriptive descriptive = this.getDescriptive(descriptiveName);

    if (descriptive != null)
    { return descriptive.getMetadata(namespace, label);
    }
    return null;
  }

  public void write(PrintWriter writer)
  { Iterator groups = this.children.values().iterator();

    while (groups.hasNext())
    { METSDescriptive group = (METSDescriptive) groups.next();

      group.write(writer);
    }
  }


  public void writeSQL(DocumentInterface document, GS3SQLConnection connection)
  { Iterator groups = this.children.values().iterator();
    
    while (groups.hasNext())
    { METSDescriptive group = (METSDescriptive) groups.next();

      group.writeSQL(document, connection);
    }
  }

  public static METSDescriptiveSet parseXML(NodeList fileSecs)
  {	
    METSDescriptiveSet set = new METSDescriptiveSet();

    for (int g = 0; g < fileSecs.getLength(); g ++) {
      //				Schema schema = new Schema(schemas.item(s));
      METSDescriptive metadata = METSDescriptive.parseXML((Element) fileSecs.item(g));
      set.addDescriptive(metadata);
    }
    return set;
  }
}

