package org.greenstone.gsdl3.gs3build.metadata;

import java.io.PrintWriter;

import java.util.List;

import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;
import org.greenstone.gsdl3.gs3build.database.*;

/**
 *  Implement a single namespace
 */

public abstract class METSNamespace
{
  protected String       type;        // the type of the metadata
  protected String       otherType;   // if a non-standard type, then...
  protected String       name;        // e.g. "DC" for Dublin Core
  protected METSLocation location;    // the location of the corresponding file
  protected String       group;       // the group of metadata namespaces to which
  // this namespace belongs
  protected String       id;          // the ID for this namespace

  public METSNamespace(String name)
  { this.name        = name;
    this.location    = null;
    this.group       = null;
    this.id          = null;
  }

  public METSNamespace(String name, METSLocation location)
  { this.location = location;
    this.name = name;
    this.group       = null;
    this.id          = null;
  }

	/**
	 *  Get the name of the namespace...
	 *
	 *  @return <code>String</code> the name of the namespace
	 */
	public String getName()
	{	return this.name;
	}

	/**
	 *  Get the group of namespaces to which this namespace belongs.
	 *
	 *  @return <code>String</code> the name of the namespace group
	 */
	public String getGroup()
	{	return this.group;
	}
	
	/**
	 *  Set the group of namespaces to which this namespace belongs.
	 *
	 *  @param <code>String</code> the name of the namespace group
	 */
	public void setGroup(String group)
	{	this.group = group;
	}

	/**
	 *  Get the ID for this namespace.
	 *
	 *  @return <code>String</code> the id of the namespace
	 */
	public String getID()
	{ return this.id;
	}

	/**
	 *  Set the id of this namespace.
	 *
	 *  @param <code>String</code> the name of the namespace group
	 */
	public void setID(String id)
	{	this.id = id;
	}

  /**
   *  <p>Indicate whether this namespace is open to being changed or not.</p>
   *  <p>Metadata which is created from a distinct file cannot be changed,
   *  only those which have no associated file can be modified.
   *
   *	@return <code>boolean</code> whether this namespace can be altered.
   */
  public boolean isEditable()
  { return (this.location == null);
  }

  public abstract boolean addMetadata(String label, String value);
  public abstract boolean setMetadata(String label, String value);
  public abstract boolean removeMetadata(String label);
  public abstract boolean removeMetadata(String label, String value);
  public abstract List    getMetadata(String label);
  public abstract boolean write(PrintWriter writer);
  //  public abstract boolean writeSQL(GS3SQLConnection connection);

  /**
   *  Write out the metadata to an SQL database through a <code>GS3SQLConnection</code>.
   *
   *  @param <code>GS3SQLConnection</code> the SQL database to use.
   */
  public boolean writeSQL(int parentId, GS3SQLConnection connection)
  { 
    GS3SQLInsert insert = new GS3SQLInsert("namespaces");

    if (this.id != null) {
	insert.addValue("NamespaceID", this.id);
    }

    if (this.location != null) {
      insert.addValue("fileLoc", this.location.getLocation().toString());
      insert.addValue("fileType", "URL");
    }
    else {
      // no location stuff
    }
    insert.addValue("MetadataRef", Integer.toString(parentId), GS3SQLField.INTEGER_TYPE);
    insert.addValue("NamespaceType", this.name);

    //    System.out.println(insert.toString());
    connection.execute(insert.toString());

    return true;
  }
}
