package org.greenstone.gsdl3.gs3build.metadata;

import java.io.PrintWriter;

import java.util.List;
import java.util.ArrayList;
import java.util.Map;
import java.util.HashMap;
import java.util.Iterator;

import java.sql.SQLException;

import org.greenstone.gsdl3.gs3build.doctypes.DocumentInterface;

import org.greenstone.gsdl3.gs3build.util.XMLTools;
import org.greenstone.gsdl3.gs3build.util.GS3SQLConnection;
import org.greenstone.gsdl3.gs3build.database.*;

public class METSStructure
{
	Map children;
	String ID;
	String label;
	String type;

	public METSStructure(String id, String label, String type)
	{	this.children = new HashMap();
		this.ID       = id;
		this.label    = label;
		this.type     = type;
	}

	public String getID()
	{	return this.ID;
	}

	public void addDivision(METSDivision division)
	{	this.children.put(division.getID(), division);
	}

	public void addSubDivision(String divisionID, METSDivision division)
	{	String topDivision = METSDivision.getTopDivisionName(divisionID);

		METSDivision child = (METSDivision) this.children.get(topDivision);
		if (child == null)
		{	return;
		}

		String subDivisionID = METSDivision.getSubDivisionName(divisionID);
		if (subDivisionID == null)
		{ child.addSubDivision(division);
		}
		else
		{	child.addSubDivision(subDivisionID, division);
		}
	}

	public METSDivision getDivision(String divisionID)
	{	String topDivision = METSDivision.getTopDivisionName(divisionID);

		METSDivision child = (METSDivision) this.children.get(topDivision);
		if (child == null)
		{	return null;
		}

		String subDivision = METSDivision.getSubDivisionName(divisionID);
		if (subDivision != null)
		{	child = child.getDivision(subDivision);
		}

		return child;
	}

	public void write(PrintWriter writer)
	{ Iterator groups = this.children.values().iterator();

		String tag = XMLTools.getOpenTag("mets", "structMap");
		tag = XMLTools.addAttribute(tag, "ID", this.ID.toString());
		tag = XMLTools.addAttribute(tag, "TYPE", this.type.toString());
		tag = XMLTools.addAttribute(tag, "LABEL", this.label.toString());
		
		writer.println(tag);
		while (groups.hasNext())
		{	METSDivision group = (METSDivision) groups.next();

			group.write(writer);
		}
		writer.println("</mets:structMap>");
	}


    public void writeSQL(DocumentInterface document, GS3SQLConnection connection)
    { Iterator groups = this.children.values().iterator();

      GS3SQLInsert insert = new GS3SQLInsert("structure");
      insert.addValue("DocID", document.getID().toString());
      insert.addValue("StructureID", this.ID.toString());
      insert.addValue("Type", this.type.toString());
      insert.addValue("Label", this.label.toString());
      
      System.out.println(insert.toString());
      connection.execute(insert.toString());
      
      // get the new structure reference
      GS3SQLSelect select = new GS3SQLSelect("structure");
      select.addField("StructureRef");
      GS3SQLWhereItem whereItem = new GS3SQLWhereItem("StructureID", "=", this.ID);
      GS3SQLWhereItem item = new GS3SQLWhereItem("DocID", "=", document.getID().toString());
      GS3SQLWhere where = new GS3SQLWhere(whereItem);
      where.add(item);
      select.setWhere(where);
      System.out.println(select.toString());
      connection.execute(select.toString());

      try {
        // read in the structure reference
        connection.getResultSet().first();
	int sqlRef = connection.getResultSet().getInt("StructureRef");
        
	while (groups.hasNext())
	{ METSDivision group = (METSDivision) groups.next();

          group.writeSQL(sqlRef, true, connection);
	}
      }
      catch (SQLException sql) {
	  System.out.println(sql);
      }
    }
}
