package org.greenstone.gsdl3.gs3build.util;

import java.util.List;
import java.util.ArrayList;

import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.sql.Statement;
import java.sql.ResultSet;


import org.greenstone.gsdl3.gs3build.database.*;

public class GS3SQLConnection
{
  private Connection connection;
  private Statement  statement;

  class GS3SQLCreateTable
  { String tableName;
    List   properties;
    List   primaryKey;
    List   indexed;

    public GS3SQLCreateTable(String tableName)
    { this.tableName = tableName;
      this.properties = new ArrayList();
      this.primaryKey = new ArrayList();
      this.indexed    = new ArrayList();
    }

    public void addAutoPrimaryKey(String fieldName)
    {
      GS3SQLField field = new GS3SQLField(fieldName, GS3SQLField.AUTOINTEGER_TYPE);
      this.properties.add(field);
      this.setPrimaryKey(fieldName);
    }

    public void addProperty(String fieldname)
    { GS3SQLField field = new GS3SQLField(fieldname);
      this.properties.add(field);
    }
    
    public void addProperty(String fieldname, String type)
    { GS3SQLField field = new GS3SQLField(fieldname, type);
      this.properties.add(field);
    }

    public void addProperty(String fieldname, int length)
    { GS3SQLField field = new GS3SQLField(fieldname, length);
      this.properties.add(field);
    }

    public void setPrimaryKey(String fieldname)
    { this.primaryKey.clear();
      this.primaryKey.add(fieldname);
    }

    public void extendPrimaryKey(String fieldname)
    { this.primaryKey.add(fieldname);
    }
    
    public void addIndex(String fieldname)
    { this.indexed.add(fieldname);
    }

    public String toString()
    {	
      StringBuffer reply = new StringBuffer();

      reply.append("CREATE TABLE ");
      reply.append(this.tableName);
      reply.append(" (");
      for (int f = 0; f < this.properties.size(); f ++)
      { GS3SQLField field = (GS3SQLField) this.properties.get(f);
	if (f != 0)
	{ reply.append(",");
	}
	reply.append(field.toString());
      }

      for (int p = 0; p < this.primaryKey.size(); p ++)
      { if (p == 0)
	{ reply.append(", PRIMARY KEY(");
	}
        else
	{ reply.append(",");
	}
        reply.append(this.primaryKey.get(p).toString());

	if (p == this.primaryKey.size() - 1)
	{ reply.append(")");
	}
      }
      // TODO: add primary key, index etc. notations
      reply.append(");");
      
      return reply.toString();
    }
  }

  public GS3SQLConnection(java.sql.Connection connection)
  { this.connection = connection;
  }

  public boolean execute(String sql)
  {
    try {
      this.statement = this.connection.createStatement();
      this.statement.execute(sql);
    }
    catch (SQLException ex) {
	System.out.println(ex);
      return false;
    }
    return true;
  }

  public Statement getStatement()
  { return this.statement;
  }

  public ResultSet getResultSet()
  { try {
      return this.statement.getResultSet();
    }
    catch (SQLException ex)
    {
      return null;
    }
  }

  public void initCollection(String collectionName)
  {	
    // Check if the collection already exists
    // if so, abort
    // else
    //   create database
    String command = "CREATE DATABASE " + collectionName;

    try {
    this.statement = this.connection.createStatement();
    this.statement.execute(command);

    // TODO: reconnect with the new database
    this.connection = GS3SQLConnectionFactory.reconnect(collectionName);

    //   create document table
    GS3SQLCreateTable docTable = new GS3SQLCreateTable("document");
    docTable.addProperty("DocID", 64);
    docTable.setPrimaryKey("DocID");
    statement = this.connection.createStatement();
    System.out.println(docTable.toString());
    statement.execute(docTable.toString());
    
    //   create structure table
    GS3SQLCreateTable structureMap = new GS3SQLCreateTable("structure");
    structureMap.addAutoPrimaryKey("StructureRef");
    structureMap.addProperty("DocID", 64);        // a 'foreign key' in effect - but not treated
    // as such for efficiency reasons
    structureMap.addProperty("StructureID", 64);  // this identifier
    structureMap.addProperty("Type", 64);         // the type of the structure
    structureMap.addProperty("Label", 128);       // the label of the structure
    statement.execute(structureMap.toString());
    
    //   create section table
    GS3SQLCreateTable sectionMap = new GS3SQLCreateTable("divisions");
    sectionMap.addAutoPrimaryKey("divisionRef");
    sectionMap.addProperty("StructureRef", GS3SQLField.INTEGER_TYPE);
    sectionMap.addProperty("DocID", 64);        // a 'foreign key' in effect - but not treated
    // as such for efficiency reasons
    sectionMap.addProperty("ParentType", 64); 	// the type of the parent - document or section
    sectionMap.addProperty("ParentRef", 64);   	// the parent sql reference identifier
    sectionMap.addProperty("SectionID", 64);  	// this identifier
    sectionMap.addProperty("Type", 64);       	// the type of the section
    sectionMap.addProperty("LabelOrder", 64);   // the order of the section
    sectionMap.addProperty("ShortLabel", 128);  // the short label of the section
    sectionMap.addProperty("UserLabel", 255);	// the long label of the section
    System.out.println(sectionMap.toString());
    statement.execute(sectionMap.toString());

    // a division->metadata reference mapping
    GS3SQLCreateTable metaRefs = new GS3SQLCreateTable("divisionmetarefs");
    metaRefs.addProperty("divisionRef", GS3SQLField.INTEGER_TYPE);
    metaRefs.addProperty("Type", 16);        // whether a namespace or meta ref
    metaRefs.addProperty("metadataRef", 32); // the metadata reference itself, as a string
    statement.execute(metaRefs.toString());

    // a division->file reference mapping
    GS3SQLCreateTable fileRefs = new GS3SQLCreateTable("divisionfilerefs");
    fileRefs.addProperty("divisionRef", GS3SQLField.INTEGER_TYPE);
    fileRefs.addProperty("Type", 16); // whether section, group or file
    fileRefs.addProperty("fileRef", 32);
    statement.execute(fileRefs.toString());
    
    //   create metadata table section table
    GS3SQLCreateTable metadataSection = new GS3SQLCreateTable("metadata");
    metadataSection.addAutoPrimaryKey("MetadataRef");
    metadataSection.addProperty("DocID", 64);   // this table isn't actually used for much
    metadataSection.addProperty("MetaID", 64);  // in and of itself...
    System.out.println(metadataSection.toString());
    statement.execute(metadataSection.toString());
    
    //   create namespace table
    GS3SQLCreateTable namespaces = new GS3SQLCreateTable("namespaces");
    namespaces.addAutoPrimaryKey("NamespaceRef");
    namespaces.addProperty("MetadataRef", GS3SQLField.INTEGER_TYPE);
    namespaces.addProperty("DocID", 64);   // these three fields are the primary key
    namespaces.addProperty("MetaID", 64);
    namespaces.addProperty("NamespaceID", 64);
    namespaces.addProperty("NamespaceType", 64);
    namespaces.addProperty("fileType", 64);
    namespaces.addProperty("fileLoc");
    namespaces.addProperty("creator", 128);
    statement.execute(namespaces.toString());
    
    //   create metadata values table
    GS3SQLCreateTable metadata = new GS3SQLCreateTable("mdvalues");
    // todo: some unique id for the namespaces item
    metadata.addProperty("NamespaceRef", GS3SQLField.INTEGER_TYPE);
    metadata.addProperty("label", 256);
    metadata.addProperty("value");
    statement.execute(metadata.toString());
    
    //   create file section table
    GS3SQLCreateTable filesec = new GS3SQLCreateTable("filesection");
    filesec.addProperty("DocID", 64);
    filesec.addProperty("FileSecID", 64);
    statement.execute(filesec.toString());
    
    //   create file groups table
    GS3SQLCreateTable filegroups = new GS3SQLCreateTable("filegroups");
    // TODO: some unique identifier
    filegroups.addProperty("DocID", 64);
    filegroups.addProperty("FileGroupID", 64);
    filegroups.addProperty("ParentGroup", 64);
    statement.execute(filegroups.toString());
    
    //   create file table
    GS3SQLCreateTable files = new GS3SQLCreateTable("files");
    // TODO: the unique identifier from file groups table
    files.addProperty("FileLocType", 64);
    files.addProperty("FileLocation");
    files.addProperty("MIMEType", 64);
    files.addProperty("ID", 32);
    statement.execute(files.toString());
    
    //   create admin table...etc.
    }
    catch (SQLException ex)
      {
	  System.out.println(ex.toString());
      }
  }

  public void deleteCollection(String collection)
  {
    try {
	statement = this.connection.createStatement();
	statement.execute("DROP DATABASE "+collection+";");
    }
    catch (SQLException ex)
      {
      }
  }
}


